package me.teridax.jcash.decode;

import org.junit.Test;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.Objects;
import java.util.regex.Pattern;

import static junit.framework.TestCase.assertEquals;

public class Decoder {

    private static final Locale LOCALE = Locale.GERMANY;
    private static final NumberFormat LOCAL_NUMBER_FORMAT = NumberFormat.getInstance(LOCALE);

    public static double decodePercent(String number) throws IllegalArgumentException, NullPointerException {
        Objects.requireNonNull(number);

        // trim the number and cut out optional percent symbols
        var trimmed = number.trim().replace("%", "");

        try {
            return LOCAL_NUMBER_FORMAT.parse(trimmed).doubleValue();
        } catch (ParseException ex) {
            throw new IllegalArgumentException("Not a valid number: " + number, ex);
        }
    }

    public static double decodeCurrency(String currency) throws IllegalArgumentException, NullPointerException {
        Objects.requireNonNull(currency);

        try {
            return LOCAL_NUMBER_FORMAT.parse(currency.trim()).doubleValue();
        } catch (ParseException ex) {
            throw new IllegalArgumentException("Not a valid currency in german locale: " + currency, ex);
        }
    }

    public static int decodeUniqueIdentificationNumber(String number) throws IllegalArgumentException, NullPointerException {
        Objects.requireNonNull(number);

        // check if the string is a valid unsigned number
        try {
            LOCAL_NUMBER_FORMAT.setParseIntegerOnly(true);
            var serialNumber = LOCAL_NUMBER_FORMAT.parse(number.trim());
            LOCAL_NUMBER_FORMAT.setParseIntegerOnly(false);
            return serialNumber.intValue();
        } catch (ParseException ex) {
            throw new IllegalArgumentException("Not a valid serial number: " + number, ex);
        }
    }

    public static String decodeName(String name) throws IllegalArgumentException, NullPointerException {
        Objects.requireNonNull(name);

        var trimmed = name.trim();

        var pattern = Pattern.compile("[\\w-\\s]+", Pattern.CASE_INSENSITIVE);
        var matcher = pattern.matcher(trimmed);
        if (matcher.find()) {
            return matcher.group();
        } else {
            throw new IllegalArgumentException("not a void name");
        }
    }

    public static String decodeStreet(String street) throws IllegalArgumentException, NullPointerException {
        Objects.requireNonNull(street);

        var pattern = Pattern.compile("\\S+(\\s+\\d+(/\\d+)?)?", Pattern.CASE_INSENSITIVE);
        var matcher = pattern.matcher(street);
        if (matcher.find()) {
            return matcher.group();
        } else {
            throw new IllegalArgumentException("not a void address");
        }
    }

    @Test
    public void testDecodeSuccessfulFunctions() {
        decodeUniqueIdentificationNumber("95786978625347895");
        decodeUniqueIdentificationNumber("    927856347   ");
        decodeUniqueIdentificationNumber("0");

        decodeName("Adolf");
        decodeName("Günther");
        decodeName("Saßkia");

        decodeStreet("Bahnhofstraße 40/1");
        decodeStreet("Gülleweg 9");
        decodeStreet("Echsengaße 67 / 4");

        assertEquals(decodePercent("1,4%"), 1.4d);
        assertEquals(decodePercent("99"), 99.0d);
        assertEquals(decodePercent("1,003  %"), 1.003d);

        assertEquals(decodeCurrency("1,3€"), 1.3d);
        assertEquals(decodeCurrency("145,34"), 145,34d);
        assertEquals(decodeCurrency("0,45   €"), 0.45d);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testDecodeInvalidFunctions() {
        decodeUniqueIdentificationNumber("q0948tvb6q047t 740   t74z0tz    784");
        decodeUniqueIdentificationNumber("-39867.8475");
        decodeUniqueIdentificationNumber("-398678475");
        decodeUniqueIdentificationNumber(" ß9qu908t76q34798t6q734vb9843");

        decodeName("John Doe");
        decodeName("3490qt67v 0b34");
        decodeName("Alexander9");
        decodeName("-ga76re78g6$§");

        decodeStreet("Bahnhofstraße -40/1");
        decodeStreet("Gülleweg 9//567");
        decodeStreet("23Echsengaße 67 / 4 Hofwetg 9");
    }
}
